<?php

/*
 * Plugin Name: GooyaPay for RCP
 * Plugin URI: https://gooyapay.ir
 * Description: GooyaPay Online Payment for RCP
 * Version: 1.0
 * Author: ماژول بانک
 * Author URI: https://www.modulebank.ir
 * Text Domain: gooyapay-payment-for-rcp
 * Domain Path: /languages
 */

if (!defined('ABSPATH')) exit;

function rcp_gooyapay_init()
{
	load_plugin_textdomain('gooyapay-payment-for-rcp', false, basename(dirname(__FILE__)) . '/languages');
	if ( !class_exists( 'RCP_Payment_Gateway' ) ) return;
	__('GooyaPay for RCP', 'gooyapay-payment-for-rcp');
	__('GooyaPay Online Payment for RCP', 'gooyapay-payment-for-rcp');
	function show_rcp_gooyapay_verify_msg($content)
	{
		if (!isset($_GET['rcp_gooyapay_verify_msg'])) return $content;
		$data = get_transient('gooyapay_rcp_' . sanitize_text_field($_GET['rcp_gooyapay_verify_msg']));
		if (!$data) return $content;
		delete_transient('gooyapay_rcp_' . sanitize_text_field($_GET['rcp_gooyapay_verify_msg']));
		return $content.$data;
		return '<br /><div class="box info-box">'.$data.'</div><br />'.$content;
	}
	add_action('the_content', 'show_rcp_gooyapay_verify_msg');
	class RCP_Payment_Gateway_GooyaPay extends RCP_Payment_Gateway
	{
		function process_signup()
		{
			global $rcp_options;
			$time = time();
			$ch = curl_init();
			curl_setopt($ch, CURLOPT_URL, 'https://gooyapay.ir/webservice/rest/PaymentRequest');
			curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($p = array(
					'MerchantID' 	=> $rcp_options['gooyapay_merchant'],
					'Amount' 		=> intval(ceil($this->initial_amount / ($this->currency == 'IRT' ? 1 : 10))),
					'InvoiceID' 	=> $time,
					'Description' 	=> $this->subscription_name,
					'CallbackURL' 	=> add_query_arg(array('listener' => 'gooyapay-rcp'), home_url('index.php'))
				)));
			curl_setopt($ch, CURLOPT_HTTPHEADER, array(
					'Content-Type: application/json',
					'Accept: application/json'
				));
			curl_setopt($ch, CURLOPT_TIMEOUT, 30);
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
			curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
			curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
			$response = curl_exec($ch);
			$error = curl_error($ch);
			curl_close($ch);
			$result = json_decode($response);
			if (isset($result->Status) && $result->Status == 100)
			{
				$data = array('key' => $this->subscription_key, 'pa' => $p['Amount'], 'amount' => $this->initial_amount, 'currency' => $this->currency, 'user_id' => $this->user_id, 'sname' => $this->subscription_name);
				set_transient('gooyapay_rcp_' . $time, $data, 1000);
				session_write_close();
				header('Location: ' . $result->PaymentUrl);
				exit;
			}
			elseif (isset($result->Message) && $result->Message != '')
			{
				echo 'Error : ' . esc_html($result->Message);
			}
			elseif ($error)
			{
				echo 'Connecting Error : ' . esc_html($error);
			}
			else
			{
				echo 'UnExpected Error in connecting to web service';
			}
			exit;
		}

		public function process_webhooks()
		{
			if (isset($_GET['listener'],$_POST['PaymentStatus'],$_POST['Authority'],$_POST['InvoiceID']) && sanitize_text_field($_GET['listener']) == 'gooyapay-rcp')
			{
				$PaymentStatus = sanitize_text_field($_POST['PaymentStatus']);
				$InvoiceID = sanitize_text_field($_POST['InvoiceID']);
				$Authority = sanitize_text_field($_POST['Authority']);
				if ($PaymentStatus == 'OK')
				{
					global $rcp_options, $wpdb, $rcp_payments_db_name;
					$data = get_transient('gooyapay_rcp_' . $InvoiceID);
					delete_transient('gooyapay_rcp_' . $InvoiceID);
					$user_id = intval($data['user_id']);
					$member = new RCP_Member($user_id);
					if(!($member && $member->ID > 0)) die( __('no member found', 'gooyapay-payment-for-rcp') );
					$subscription_id = $member->get_pending_subscription_id();
					if( empty( $subscription_id ) ) $subscription_id = $member->get_subscription_id();
					if( ! $subscription_id ) die( __('no subscription for member found', 'gooyapay-payment-for-rcp') );
					if( ! $x = rcp_get_subscription_details( $subscription_id ) ) die( __('no subscription level found', 'gooyapay-payment-for-rcp') );
					$pending_amount = get_user_meta( $member->ID, 'rcp_pending_subscription_amount', true );
					$ch = curl_init();
					curl_setopt($ch, CURLOPT_URL, 'https://gooyapay.ir/webservice/rest/PaymentVerification');
					curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode(array(
							'MerchantID' 	=> $rcp_options['gooyapay_merchant'],
							'Amount' 		=> $data['pa'],
							'Authority' 	=> $Authority
						)));
					curl_setopt($ch, CURLOPT_HTTPHEADER, array(
							'Content-Type: application/json',
							'Accept: application/json'
						));
					curl_setopt($ch, CURLOPT_TIMEOUT, 30);
					curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
					curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
					curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
					$response = curl_exec($ch);
					curl_close($ch);
					$result = json_decode($response);
					if (isset($result->Status) && in_array($result->Status, array(100,101)))
					{
						$has_trial            = false;
						$payment_status       = 'completed';
						$subscription_key     = $data['key'];
						$currency_code        = $this->currency?:rcp_get_currency();
						$pending_amount       = number_format( (float) $pending_amount, 2, '.', '' );
						$pending_payment_id   = $member->get_pending_payment_id();
						delete_user_meta( $member->ID, 'rcp_pending_subscription_amount' );
						$rcp_payments = new RCP_Payments();
						if($rcp_payments->payment_exists($result->RefID)) die( __('duplicate Payment detected', 'gooyapay-payment-for-rcp') );
						$payment_data = array(
								'date'             => date( 'Y-m-d H:i:s', strtotime( 'now', current_time( 'timestamp' ) ) ),
								'subscription'     => $data['sname'],
								'payment_type'     => 'web_accept',
								'subscription_key' => $subscription_key,
								'amount'           => $pending_amount,
								'user_id'          => $user_id,
								'transaction_id'   => $result->RefID,
								'status'           => 'complete'
							);
						if( $member->just_upgraded() && $member->can_cancel() )
						{
							$cancelled = $member->cancel_payment_profile( false );
							if( $cancelled )
							{
								$member->set_payment_profile_id( '' );
							}
						}
						if ( ! empty( $pending_payment_id ) )
						{
							$member->set_recurring( false );
							$rcp_payments->update( $pending_payment_id, $payment_data );
							$payment_id = $pending_payment_id;
						}
						else
						{
							$member->renew();
							$payment_id = $rcp_payments->insert( $payment_data );
						}
						set_transient('gooyapay_rcp_' . $InvoiceID, __('Payment Completed. RefNumber : ', 'gooyapay-payment-for-rcp') . $result->RefID, 60);
						header('location: '.add_query_arg(array('rcp_gooyapay_verify_msg' => $InvoiceID), rcp_get_return_url($user_id)));
						exit;
					}
					else
					{
						if (isset($result->Message) && $result->Message != '')
						{
							$message = 'Error : ' . $result->Message;
						}
						elseif ($error)
						{
							$message = 'Connecting Error : ' . $error;
						}
						else
						{
							$message = 'UnExpected Error in connecting to web service';
						}
						set_transient('gooyapay_rcp_' . $InvoiceID, $message, 60);
						header('location: '.add_query_arg(array('rcp_gooyapay_verify_msg' => $InvoiceID), rcp_get_return_url()));
						//header('location: '.get_permalink( $rcp_options['registration_page'] ));
						echo esc_html($message);
						exit;
					}
				}
				else
				{
					$message = 'پرداخت از سوی کاربر لغو شد';
					set_transient('gooyapay_rcp_' . $InvoiceID, $message, 60);
					header('location: '.add_query_arg(array('rcp_gooyapay_verify_msg' => $InvoiceID), rcp_get_return_url()));
					//header('location: '.get_permalink( $rcp_options['registration_page'] ));
					echo esc_html($message);
					exit;
				}
			}
		}
	}
}
add_action('plugins_loaded', 'rcp_gooyapay_init', 666);

if (!function_exists('rcp_register_irt_currency'))
{
	function rcp_register_irt_currency( $currencies )
	{
		return array_merge(array('IRT' => __('Iranian Toman', 'gooyapay-payment-for-rcp')), $currencies);
	}
	add_filter('rcp_currencies', 'rcp_register_irt_currency');
	add_filter( 'rcp_is_zero_decimal_currency', function($r){
		if (in_array(strtoupper(rcp_get_currency()), array('IRR','IRT')))
			$r = true;
		return $r;
	}, 666 );
}

function rcp_register_gooyapay_payment_gateway($gateways)
{
	return array_merge(array('gooyapay' => array('label' => __('GooyaPay', 'gooyapay-payment-for-rcp'), 'admin_label' => __('GooyaPay', 'gooyapay-payment-for-rcp'), 'class' => 'RCP_Payment_Gateway_GooyaPay')),$gateways);
}
add_filter('rcp_payment_gateways', 'rcp_register_gooyapay_payment_gateway');

function rcp_show_gooyapay_setting($rcp_options)
{
	?>
	<hr/>
	<table class="form-table">
		<tr valign="top"><th colspan=2><h3> <?php _e('GooyaPay Gateway Setting', 'gooyapay-payment-for-rcp'); ?> </h3></th></tr>
		<tr>
			<th><label for="rcp_settings[gooyapay_merchant]"> <?php _e('GooyaPay Merchant Code', 'gooyapay-payment-for-rcp'); ?> </label></th>
			<td><input class="regular-text" id="rcp_settings[gooyapay_merchant]" style="width: 300px;" name="rcp_settings[gooyapay_merchant]" value="<?php echo $rcp_options['gooyapay_merchant']; ?>"/></td>
		</tr>
	</table>
	<?php
}
add_action('rcp_payments_settings', 'rcp_show_gooyapay_setting');

?>